#!/usr/bin/env python3
"""
 DESCRIPTION
          This tool is part of cygnsslib python package. The package is created by Mixil lab at USC
          See <https://bitbucket.org/usc_mixil/cygnss-library>

          This Tool download SRTM .hgt data

 AUTHOR   Amer Melebari
          Microwave Systems, Sensors and Imaging Lab (MiXiL)
          University of Southern California
 EMAIL    jamesdca@usc.edu
 CREATED  2020‑07‑19
 Updated  2020-08-22

  Copyright 2020 University of Southern California
"""
from cygnsslib.data_downloader.download_srtm import EarthdataSession
from cygnsslib.data_downloader.pass_core import MixilKeys
from getpass import getpass
from time import sleep
from tqdm.auto import tqdm
import datetime as dt
import fnmatch
import hashlib
import numpy as np
import os
import requests
import shutil
import warnings

CYG_MIN_FILE_SIZE = 50e6  # in bytes
PODAAC_CYG_URL = 'https://podaac-tools.jpl.nasa.gov/drive/files/allData/cygnss'
OPEN_DAP_URL = 'https://opendap.jpl.nasa.gov/opendap/allData/cygnss'
L1_VER = 'v3.0'
CHUNK_SIZE = 1024 * 1024  # 1 MB
MAX_CONNECTIONS_TRIES = 10


def _cyg_data_auth_err_handling():
    print('Error in the PODAAC API Credentials, try to enter them again.')
    print('Note: you need to re-start the code after this download as the new user/pass need to be re-loaded')
    save_pass_str = input('Do you want to save the new user/pass? (if yes, the old user/pass will be deleted) [Yes, No]')
    save_pass = True if (save_pass_str.lower() in ['y', 'yes']) else False
    podaac_usr, podaac_pass = get_podaac_cred(pass_folder=None, save_pass=save_pass)
    return podaac_usr, podaac_pass


def checksum(file_path, chunk_num_blocks=4096):
    """
    do md5 checksum for large files

    :param file_path: file path
    :type file_path: str
    :param chunk_num_blocks: number of blocks in a chunk
    :type chunk_num_blocks: int
    :return:
    """

    h = hashlib.md5()
    with open(file_path, 'rb') as f:
        while chunk := f.read(chunk_num_blocks * h.block_size):
            h.update(chunk)
    return h.hexdigest()


def download_file(file_url, output_folder, auth=None, url_md5_checksum=None):
    """
    download the file with url into folder output_folder

    :param file_url: url of the file
    :type file_url: str
    :param output_folder: saving folder
    :type output_folder: str
    :param auth: username or pass
    :type auth: tuple of str or None
    :param url_md5_checksum: md5 checksum of the downloaded file, if None, the code will find it
    :type url_md5_checksum: str
    :return: downloaded file path
    :rtype: str
    """
    num_redownload = 0
    file_name = file_url.split('/')[-1]
    out_file = os.path.join(output_folder, file_name)
    out_file_temp = f'{out_file:s}.incomplete'
    for i_try in range(MAX_CONNECTIONS_TRIES):
        try:
            with EarthdataSession(username=auth[0], password=auth[1]) as session:
                with session.get(file_url, stream=True) as response:
                    if response.status_code == 404:
                        return None
                    elif response.status_code == 401:  # Auth Error
                        raise requests.exceptions.RequestException
                    response.raise_for_status()
                    response.raw.decode_content = True
                    file_size = int(response.headers.get('content-length', 0))
                    free_disk_space = shutil.disk_usage(output_folder).free
                    if file_size > free_disk_space:
                        bytes2mb = lambda x: int(x / 1024 / 1024)
                        raise IOError(f'No enough space in the disk. file size: {bytes2mb(file_size):d} MB, free space {bytes2mb(free_disk_space):d}')
                    with tqdm.wrapattr(open(out_file_temp, "wb"), "write", miniters=1, total=file_size, desc=out_file_temp) as target_file:
                        for chunk in response.iter_content(chunk_size=CHUNK_SIZE):
                            target_file.write(chunk)
                if url_md5_checksum is None:  # get md5 checksum
                    url_md5_checksum = session.get(f'{file_url:s}.md5').content.strip().decode('utf').split()[0]
                if url_md5_checksum:  # if the md5 file available
                    file_checksum = checksum(out_file_temp)
                    if url_md5_checksum != file_checksum:
                        raise RuntimeError
                else:
                    print(f'MD5 checksum is not available for this file; {file_name:s}')
        except RuntimeError:  # MD5 verification failed
            if i_try < MAX_CONNECTIONS_TRIES and num_redownload < 2:
                print("MD5 checksum failed! Trying to re-download it again")
                os.remove(out_file_temp)
                num_redownload += 1
            else:
                print("MD5 checksum failed!")
                break
        except (requests.exceptions.ConnectionError, requests.exceptions.HTTPError):
            if i_try < MAX_CONNECTIONS_TRIES:
                print('Connection Error sleep for 5 sec')
                sleep(5)
            else:
                raise RuntimeError(f'a connection error was raised when trying to access {file_url:s}')
        except requests.exceptions.RequestException:
            if i_try < MAX_CONNECTIONS_TRIES:
                podaac_usr, podaac_pass = _cyg_data_auth_err_handling()
                auth = (podaac_usr, podaac_pass)
            else:
                raise RuntimeError('Authentication error, please check your PODAAC user/pass')

        else:  # if it was successful
            break

    shutil.move(out_file_temp, out_file)
    return out_file


def _get_cyg_page_data(cyg_folder_url, podaac_usr_pass):
    data = None
    for i_try in range(MAX_CONNECTIONS_TRIES):
        try:
            with EarthdataSession(username=podaac_usr_pass[0], password=podaac_usr_pass[1]) as session:
                with session.get(cyg_folder_url) as response:
                    data = str(response.content).split('<tr>')
        except (requests.exceptions.ConnectionError, requests.exceptions.HTTPError):
            if i_try < MAX_CONNECTIONS_TRIES:
                print('Connection Error sleep for 5 sec')
                sleep(5)
            else:
                raise RuntimeError(f'a connection error was raised when trying to access {cyg_folder_url:s}')

        else:  # if it was successful
            break

    return data


def _get_md5_file(day_page_data, sc_num, cyg_folder_url, auth):
    tag = f'cyg{sc_num:02d}.'
    endtag = '.nc.md5'
    for item in day_page_data:
        if tag in item:
            try:
                ind = item.index(tag)
                item = item[ind:]
                end = item.index(endtag)
            except ValueError:
                continue

            file_name = item[:end] + endtag
            md5_file_full_url = f'{cyg_folder_url:s}{file_name:s}'
            for i_try in range(MAX_CONNECTIONS_TRIES):
                try:
                    with EarthdataSession(username=auth[0], password=auth[1]) as session:
                        mdf5_file = session.get(md5_file_full_url).content.strip().decode('utf').split()
                        if mdf5_file:
                            cyg_file_name = mdf5_file[1]
                            cyg_checksum = mdf5_file[0]
                            return cyg_file_name, cyg_checksum
                        else:
                            print(f'MD5 checksum is not available for this file; {file_name:s}')
                            return None, None
                except (requests.exceptions.ConnectionError, requests.exceptions.HTTPError):
                    if i_try < MAX_CONNECTIONS_TRIES:
                        print('Connection Error sleep for 5 sec')
                        sleep(5)
                    else:
                        raise RuntimeError(f'a connection error was raised when trying to access {md5_file_full_url:s}')
                except requests.exceptions.RequestException:
                    if i_try < MAX_CONNECTIONS_TRIES:
                        podaac_usr, podaac_pass = _cyg_data_auth_err_handling()
                        auth = (podaac_usr, podaac_pass)
                    else:
                        raise RuntimeError('Authentication error, please check your PODAAC user/pass')

    return None, None


def download_cyg_file(data_year, data_day, sc_num, cyg_data_ver, cyg_data_lvl, out_folder, podaac_usr_pass, day_page_data=None, cyg_file_name=None,
                      cyg_md5_checksum=None):
    """
    download cygnss data file from PODAAC, the username and pass can be obtained from https://podaac-tools.jpl.nasa.gov/drive/
    Don't use this function directly, instead use download_cyg_files()

    :param data_year: data year (ex. 2019)
    :type data_year: int
    :param data_day: data day number (ex. 130)
    :type data_day: int
    :param sc_num: cygnss spacecraft number (1-8)
    :type sc_num: int
    :param cyg_data_ver: cygnss data version (ex: 'v2.1')
    :type cyg_data_ver: str
    :param cyg_data_lvl: cygnss data level (ex: 'L1')
    :type cyg_data_lvl: str
    :param out_folder: folder of the saved output (ex: /cygnss_data/L1/v2.1/2019/020)
    :type out_folder: str
    :param podaac_usr_pass: PODAAC user name and pass (usr name, pass)
    :type podaac_usr_pass: tuple of string
    :param day_page_data: page of the selected day, this to reduce the times the script requests the page from the website, default None
    :type day_page_data: list of str or None
    :param cyg_file_name: cygness file name, if None, the code will find it
    :type cyg_file_name: str
    :param cyg_md5_checksum: md5 checksum, if None, the code will find it
    :type cyg_md5_checksum: str
    :return: file name
    :rtype: str or None
    """

    if os.path.isfile(out_folder):
        raise ValueError(f'Invalid out_folder={out_folder}, this is a file not a folder')

    cyg_folder_url = f'{PODAAC_CYG_URL:s}/{cyg_data_lvl:s}/{cyg_data_ver:s}/{data_year:04d}/{data_day:03d}/'
    if cyg_file_name is None:
        if day_page_data is None:
            day_page_data = _get_cyg_page_data(cyg_folder_url, podaac_usr_pass)

        tag = f'cyg{sc_num:02d}.'
        endtag = '.nc'
        for item in day_page_data:
            if tag in item:
                try:
                    ind = item.index(tag)
                    item = item[ind:]
                    end = item.index(endtag)
                except ValueError:
                    continue

                cyg_file_name = item[:end + 3]
                break
    if cyg_file_name is None:
        print(f"File doesn't exist in the PODAAC, year:{data_year:04d}, day:{data_day:03d}, SC: {sc_num:02d}")
        return cyg_file_name
    if not os.path.isdir(out_folder):
        os.makedirs(out_folder, exist_ok=True)

    cyg_file_full_url = f'{cyg_folder_url:s}{cyg_file_name:s}'
    cyg_file_name = download_file(cyg_file_full_url, out_folder, auth=podaac_usr_pass, url_md5_checksum=cyg_md5_checksum)
    if cyg_file_name is None:
        print(f"File doesn't exist in the PODAAC, year:{data_year:04d}, day:{data_day:03d}, SC: {sc_num:02d}")
    return cyg_file_name


def download_cyg_files(data_year, list_data_day, list_sc_num=None, podaac_usr=None, podaac_pass=None, cyg_data_ver=None, cyg_data_lvl='L1',
                       cygnss_l1_path=os.environ.get('CYGNSS_L1_PATH'), check_md5_exist_file=False, force_download=False, save_podaac_pass=True):
    """
    
    download multiple CYGNSS files
    
    :param data_year: data year
    :type data_year: int
    :param list_data_day: list of data days
    :type list_data_day: list or int or np.array
    :param list_sc_num: list of cygnss spacecraft numbers (1-8), if None will download all SCs
    :type list_sc_num: list or int or np.array or None
    :param podaac_usr: PODAAC user name. if None, it will ask you to enter it
    :type podaac_usr: str or None
    :param podaac_pass: PODAAC Drive API password. if None, it will ask you to enter it
    :type podaac_pass: str or None
    :param cyg_data_ver: cygnss data version (ex: 'v2.1')
    :type cyg_data_ver: str
    :param cyg_data_lvl: cygnss data level (ex: 'L1')
    :type cyg_data_lvl: str
    :param cygnss_l1_path: path of the cygnss L1 data (default: os.environ.get('CYGNSS_L1_PATH'))
    :type cygnss_l1_path: str or None
    :param check_md5_exist_file: check md5 checksum for existing files? this will make it very slow
    :type check_md5_exist_file: bool
    :param force_download: re-download the file even if the version is not included in the path (not recommended)
    :type force_download: bool
    :param save_podaac_pass: save podaac username and pass in your system? (select False if you're using a shared or public PC)
    :type save_podaac_pass: bool
    :return: 
    """

    if np.size(list_data_day) == 1:
        list_data_day = [int(list_data_day)]
    if list_sc_num is None:
        list_sc_num = np.arange(1, 9)
    elif np.size(list_sc_num) == 1:
        list_sc_num = [int(list_sc_num)]

    if cygnss_l1_path is None:
        raise ValueError("$CYGNSS_L1_PATH environment variable need to be set, or use cygnss_l1_path input parameter")

    if cyg_data_ver is None:
        folder_list = cygnss_l1_path.split(os.path.sep)
        if not folder_list[-1]:
            folder_list.pop(-1)
        cyg_data_ver = folder_list[-1]
    cyg_data_lvl = cyg_data_lvl.upper()

    podaac_usr_pass = get_podaac_cred(save_pass=save_podaac_pass) if (podaac_usr is None or podaac_pass is None) else (podaac_usr, podaac_pass)
    ver_folder_exist = check_ver_folder(cygnss_l1_path, cyg_data_ver)
    if not force_download and not ver_folder_exist:
        error_str = f"You are trying to download version {cyg_data_ver:s}, but the path doesn't contain the version name"
        raise ValueError(f'{error_str:s}\nuse force_download=True to remove this error')
    downloaded_files_list = list()
    for data_day in list_data_day:
        downloaded_files_list += _download_cyg_single_day(data_year, data_day, list_sc_num, podaac_usr_pass, cyg_data_ver, cyg_data_lvl,
                                                          cygnss_l1_path, check_md5_exist_file)
    return downloaded_files_list


def download_cyg_files_between_date(st_date, end_date, list_sc_num=None, podaac_usr=None, podaac_pass=None, cyg_data_ver=None, cyg_data_lvl='L1',
                                    cygnss_l1_path=os.environ.get('CYGNSS_L1_PATH'), check_md5_exist_file=False, force_download=False,
                                    save_podaac_pass=True):
    """
    download CYGNSS data between two dates (including start and end date)

    :param st_date: start date
    :type st_date: date
    :param end_date: end date
    :type end_date: date
    :param list_sc_num: list of cygnss spacecraft numbers (1-8), if None will download all SCs
    :type list_sc_num: list or np.array or int
    :param podaac_usr: PODAAC user name. if None, it will ask you to enter it
    :type podaac_usr: str or None
    :param podaac_pass: PODAAC Drive API password. if None, it will ask you to enter it
    :type podaac_pass: str or None
    :param cyg_data_ver: cygnss data version (ex: 'v2.1')
    :type cyg_data_ver: str
    :param cyg_data_lvl: cygnss data level (ex: 'L1')
    :type cyg_data_lvl: str
    :param cygnss_l1_path: path of the cygnss L1 data (default: os.environ.get('CYGNSS_L1_PATH'))
    :type cygnss_l1_path: str or None
    :param check_md5_exist_file: check md5 checksum for existing files? this will make it very slow
    :type check_md5_exist_file: bool
    :param force_download: download the file even if the version is not included in the path (not recommended)
    :param save_podaac_pass: save podaac username and pass in your system? (select False if you're using a shared or public PC)
    :type save_podaac_pass: bool
    :return:
    """

    if list_sc_num is None:
        list_sc_num = np.arange(1, 9)
    elif np.size(list_sc_num) == 1:
        list_sc_num = [int(list_sc_num)]
    if cygnss_l1_path is None:
        raise ValueError("$CYGNSS_L1_PATH environment variable need to be set, or use cygnss_l1_path input parameter")
    if cyg_data_ver is None:
        folder_list = cygnss_l1_path.split(os.path.sep)
        if not folder_list[-1]:
            folder_list.pop(-1)
        cyg_data_ver = folder_list[-1]
    cyg_data_lvl = cyg_data_lvl.upper()
    podaac_usr_pass = get_podaac_cred(save_pass=save_podaac_pass) if (podaac_usr is None or podaac_pass is None) else (podaac_usr, podaac_pass)

    ver_folder_exist = check_ver_folder(cygnss_l1_path, cyg_data_ver)
    if not force_download and not ver_folder_exist:
        error_str = f"You are trying to download version {cyg_data_ver:s}, but the path doesn't contain the ver name ({cygnss_l1_path:s})"
        raise ValueError(f'{error_str:s}\nuse force_download=True to remove this error')

    num_days = (end_date - st_date).days + 1
    downloaded_files_list = list()
    for iday in range(0, num_days):
        data_date = st_date + dt.timedelta(days=iday)
        data_year = data_date.year
        data_day = data_date.timetuple().tm_yday
        downloaded_files_list += _download_cyg_single_day(data_year, data_day, list_sc_num, podaac_usr_pass, cyg_data_ver, cyg_data_lvl,
                                                          cygnss_l1_path, check_md5_exist_file)
    return downloaded_files_list


def _download_cyg_single_day(data_year, data_day, list_sc_num, podaac_usr_pass, cyg_data_ver, cyg_data_lvl, cygnss_l1_path,
                             check_md5_exist_file=False, silent_run=False):
    """
    download a single day, access this function from download_cyg_files()
    :param data_year: data year
    :type data_year: int
    :param data_day: data day of the year
    :type data_day: int
    :param list_sc_num: list of cygnss spacecraft numbers (1-8), if None will download all SCs
    :type list_sc_num: list or np.array or int
    :param podaac_usr_pass: PODAAC user name and pass (usr name, pass)
    :type podaac_usr_pass: tuple of string
    :param cyg_data_ver: cygnss data version (ex: 'v2.1')
    :type cyg_data_ver: str
    :param cyg_data_lvl: cygnss data level (ex: 'L1')
    :type cyg_data_lvl: str
    :param cygnss_l1_path: path of the cygnss L1 data
    :type cygnss_l1_path: str
    :param check_md5_exist_file: check md5 checksum for existing files? this will make it very slow
    :type check_md5_exist_file: bool
    :param silent_run: if true no screen output will be shown except downloading progress and errors
    :type silent_run: bool
    :return:
    """

    cyg_day_folder = os.path.join(cygnss_l1_path, f'{data_year:04d}', f'{data_day:03d}')
    # if not os.path.isdir(cyg_day_folder):  # commented so we don't create folders for days without data
    #     os.makedirs(cyg_day_folder, exist_ok=True)
    cyg_day_folder_url = f'{PODAAC_CYG_URL:s}/{cyg_data_lvl:s}/{cyg_data_ver:s}/{data_year:04d}/{data_day:03d}/'
    day_page_data = _get_cyg_page_data(cyg_day_folder_url, podaac_usr_pass)
    file_name_list = list()
    for sc_num in list_sc_num:
        download_file_tf = False
        disk_cyg_file_name = get_cyg_file(cyg_day_folder, sc_num)
        cyg_md5_checksum = None
        cyg_file_name = None
        if check_md5_exist_file:
            cyg_file_name, cyg_md5_checksum = _get_md5_file(day_page_data, sc_num, cyg_day_folder_url, podaac_usr_pass)
            if cyg_file_name is None:  # No md5 ==> no cygnss file
                print(f"File doesn't exist in the PODAAC, year:{data_year:04d}, day:{data_day:03d}, sc_num: {sc_num:02d}")
                continue
            if disk_cyg_file_name is not None:
                cyg_file_full_path = os.path.join(cyg_day_folder, disk_cyg_file_name)
                # file_size = os.path.getsize(cyg_file_full_path)
                # file_name = cyg_file_name
                if cyg_file_name != disk_cyg_file_name:
                    os.remove(cyg_file_full_path)
                    download_file_tf = True
                else:
                    file_checksum = checksum(cyg_file_full_path)
                    if file_checksum != cyg_md5_checksum:
                        print('disk version of cygnss file failed md5 checksum test, re-downloading the file again')
                        os.remove(cyg_file_full_path)
                        download_file_tf = True

        file_name = None
        if not download_file_tf:
            if disk_cyg_file_name is None:
                download_file_tf = True
            else:
                cyg_file_full_path = os.path.join(cyg_day_folder, disk_cyg_file_name)
                file_size = os.path.getsize(cyg_file_full_path)
                if file_size > CYG_MIN_FILE_SIZE:
                    if not silent_run:
                        print(f'{disk_cyg_file_name:s} file exist')
                else:
                    print(f'{disk_cyg_file_name:s} file size is too small ({int(file_size/1024/1024)} MB), re-downloading the file')
                    os.remove(cyg_file_full_path)
                    download_file_tf = True

        if download_file_tf:
            file_name = download_cyg_file(data_year, data_day, sc_num, cyg_data_ver, cyg_data_lvl, cyg_day_folder, podaac_usr_pass=podaac_usr_pass,
                                          day_page_data=day_page_data, cyg_file_name=cyg_file_name, cyg_md5_checksum=cyg_md5_checksum)
        if file_name is not None:
            file_name_list.append(file_name)

    return file_name_list


def get_cyg_file(cyg_day_folder, sc_num):
    """
    check if the file exist and return the file name, if not exist return None

    :param cyg_day_folder: cygnss day folder
    :type cyg_day_folder: str
    :param sc_num: spacecraft number
    :type sc_num: int
    :return: file name
    :rtype: str
    """
    result = []
    pattern = f"cyg{sc_num:02d}*.nc"
    for root, dirs, files in os.walk(cyg_day_folder):
        for name in files:
            if fnmatch.fnmatch(name, pattern):
                result.append(name)
                break  # finding the first file

    if len(result) == 0:
        cyg_file_name = None
    else:
        cyg_file_name = result[0]

    return cyg_file_name


def check_ver_folder(cygnss_l1_path, cyg_data_ver):
    """
    check if the version name in the path

    :param cygnss_l1_path: path of the cygnss L1 data
    :type cygnss_l1_path: str
    :param cyg_data_ver: cygnss data version (ex: 'v2.1')
    :type cyg_data_ver: str
    :return:
    """
    path_split = cygnss_l1_path.split(os.sep)
    if cyg_data_ver in path_split:
        out = True
    else:
        warnings.warn(f"You are trying to download version {cyg_data_ver:s}, but the path doesn't contain the version name", RuntimeWarning)
        out = False
    return out


def get_podaac_cred(pass_folder=None, save_pass=True, reset_pass=False):
    """
    import podaac username and pass from the system, if not found it ask you to enter them

    :return:
    """
    if reset_pass:
        mixil_keys = MixilKeys(pass_folder=pass_folder)
        mixil_keys.remove('podaac_usr')
        mixil_keys.remove('podaac_pass')
        del mixil_keys

    if save_pass:
        mixil_keys = MixilKeys(pass_folder=pass_folder)
        mixil_keys.require('podaac_usr', msg='Get PODAAC API Credentials from: https://podaac-tools.jpl.nasa.gov/drive/ \nPO.DAAC Username: ')
        mixil_keys.require('podaac_pass', msg='PO.DAAC Drive API Password: ')
        podaac_usr = mixil_keys.retrieve('podaac_usr')
        podaac_pass = mixil_keys.retrieve('podaac_pass')
    else:
        podaac_usr = getpass('PO.DAAC Username: ')
        podaac_pass = getpass('PO.DAAC Drive API Password: ')

    return podaac_usr, podaac_pass


if __name__ == "__main__":
    # main()
    # from cygnsslib import cygnss_download
    # import datetime as dt

    # st_date = dt.date(year=2020, month=7, day=1)
    # end_date = dt.date(year=2020, month=12, day=31)
    # cyg_data_ver = 'v2.1'
    # download_cyg_files_between_date(st_date, end_date, cyg_data_ver=cyg_data_ver)
    os.environ["CYGNSS_L1_PATH"] = '/media/amer/Data/cygnss_data/L1/v3.0'
    st_date = dt.date(year=2019, month=1, day=1)
    end_date = dt.date(year=2019, month=12, day=31)
    cyg_data_ver = 'v3.0'
    download_cyg_files_between_date(st_date, end_date, cyg_data_ver=cyg_data_ver, check_md5_exist_file=True)



from cygnsslib.cyg import get_list_ddm_info_from_kml, cyg_environ_check
from netCDF4 import Dataset
import copy
import fnmatch
import numpy as np
import os
import pandas as pd
import warnings

L1_LAND_FOLDER = "v3Land"
L1_OCEAN_FOLDER = "v3.0"


def get_sample_id_of_different_cyg_version(in_cygddmid, new_version, original_cyg_ver=None):
    """


    :param in_cygddmid:
    :type in_cygddmid: CygDdmId
    :param new_version:
    :param original_cyg_ver:
    :return:
    """
    cyg_environ_check()
    if original_cyg_ver is not None:
        original_l1_path = os.path.join(os.environ.get("CYGNSS_PATH"), 'L1', original_cyg_ver)
    else:
        original_l1_path = os.environ.get("CYGNSS_L1_PATH")
    dist_cygnss_l1_path = os.path.join(os.environ.get("CYGNSS_PATH"), 'L1', new_version)
    in_cygddmid.fill_file_name(cygnss_l1_path=original_l1_path)
    cygnss_original_ver_file = os.path.join(original_l1_path, f'{in_cygddmid.year:04d}', f'{in_cygddmid.day:03d}', in_cygddmid.file_name)
    ds_cyg = Dataset(cygnss_original_ver_file, 'r')
    ds_cyg.set_auto_maskandscale(False)
    tcs = pd.Timestamp(ds_cyg.time_coverage_start)
    tcs_in_sec = (tcs - copy.deepcopy(tcs).replace(hour=0, minute=0, second=0, microsecond=0, nanosecond=0)).delta * 1e-9

    # Number of seconds relative to the beginning of the day [sec]. resolution is nano second
    sel_sample_time_sec = ds_cyg.variables['ddm_timestamp_utc'][in_cygddmid.samp_id] + tcs_in_sec
    ds_cyg.close()  # close the file

    # open the land file
    cyg_dis_ver_file_name = find_cygnss_file(data_year=in_cygddmid.year, data_day=in_cygddmid.day, sc_num=in_cygddmid.sc_num,
                                             cygnss_l1_path=dist_cygnss_l1_path)
    dis_ver_file_path = os.path.join(dist_cygnss_l1_path, f'{in_cygddmid.year:04d}', f'{in_cygddmid.day:03d}', cyg_dis_ver_file_name)
    dis_ver_sample_id = get_sample_id_from_time_rltv_tcs(dis_ver_file_path, sel_sample_time_sec)
    dis_ver_cygddmid = CygDdmId(cyg_dis_ver_file_name, in_cygddmid.year, in_cygddmid.day, in_cygddmid.sc_num, in_cygddmid.ch_id, dis_ver_sample_id)
    return dis_ver_cygddmid


def get_land_prod_info_from_ocean_prod(year, day, sc_num, l1_ocean_sample, cygnss_l1_path=None, l1_ocean_folder_name=None,
                                       l1_land_folder_name=None):
    """
    get sample id for land product (v3Land) from the ocean product

    :param year: data year, ex. 2019
    :type year: int
    :param day: day of the year, (1-36X)
    :type day: int
    :param sc_num: spacecraft id, (1-8)
    :type sc_num: int
    :param l1_ocean_sample: sample id of the ocean product (zero-based)
    :type l1_ocean_sample: int
    :param cygnss_l1_path: path of the cygnss L1 data (default: os.environ.get("CYGNSS_L1_PATH"))
    :type cygnss_l1_path: str or None
    :param l1_ocean_folder_name: L1 ocean folder name (default: L1_OCEAN_FOLDER)
    :type l1_ocean_folder_name: str or None
    :param l1_land_folder_name: L1 land folder name (default: L1_LAND_FOLDER)
    :type l1_land_folder_name: str or None
    :return: sample id of the land product (zero-based)
    """
    if cygnss_l1_path is None:
        raise ValueError("$CYGNSS_L1_PATH environment variable need to be set, or use cygnss_l1_path input parameter")

    l1_land_folder_name = L1_LAND_FOLDER if l1_land_folder_name is None else l1_land_folder_name
    l1_ocean_folder_name = L1_OCEAN_FOLDER if l1_ocean_folder_name is None else l1_ocean_folder_name

    cygnss_l1_root_path = uppath(cygnss_l1_path, 1)
    cygnss_l1_land_path = os.path.join(cygnss_l1_root_path, l1_land_folder_name)
    cygnss_l1_ocean_path = os.path.join(cygnss_l1_root_path, l1_ocean_folder_name)

    #  get selected DDM time stamp from the ocean file
    cyg_ocean_file_name = find_cygnss_file(data_year=year, data_day=day, sc_num=sc_num, cygnss_l1_path=cygnss_l1_ocean_path)
    cygnss_ocean_file = os.path.join(cygnss_l1_ocean_path, '{:04d}'.format(year), '{:03d}'.format(day), cyg_ocean_file_name)
    ds_cyg = Dataset(cygnss_ocean_file, 'r')
    ds_cyg.set_auto_maskandscale(False)

    tcs = pd.Timestamp(ds_cyg.time_coverage_start)
    tcs_in_sec = (tcs - copy.deepcopy(tcs).replace(hour=0, minute=0, second=0, microsecond=0, nanosecond=0)).delta * 1e-9
    sel_sample_time = np.array(ds_cyg.variables['ddm_timestamp_utc'][l1_ocean_sample]) + tcs_in_sec  # Number of seconds relative to the beginning of
    # the day [sec]. resolution is nano second

    ds_cyg.close()  # close the file

    # open the land file
    cyg_land_file_name = find_cygnss_file(data_year=year, data_day=day, sc_num=sc_num, cygnss_l1_path=cygnss_l1_land_path)
    cygnss_land_file = os.path.join(cygnss_l1_land_path, '{:04d}'.format(year), '{:03d}'.format(day), cyg_land_file_name)
    l1_land_sample_id = get_sample_id_from_time_rltv_tcs(cygnss_land_file, sel_sample_time)
    return cyg_land_file_name, l1_land_sample_id, sel_sample_time


def find_cygnss_file(data_year, data_day, sc_num, cygnss_l1_path=os.environ.get("CYGNSS_L1_PATH")):
    """
    find cygnss file name in the folder, raise FileNotFoundError if not found

    :param data_year: data year, ex. 2019
    :type data_year: int
    :param data_day: day of the year, (1-36X)
    :type data_day: int
    :param sc_num: spacecraft id, (1-8)
    :type sc_num: int
    :param cygnss_l1_path: path of the cygnss L1 data (default: os.environ.get("CYGNSS_L1_PATH"))
    :type cygnss_l1_path: str or None
    :return: cygnss file name
    :type: str
    """
    if cygnss_l1_path is None:
        cygnss_l1_path = os.environ.get("CYGNSS_L1_PATH")
        if cygnss_l1_path is None:
            raise ValueError("$CYGNSS_L1_PATH environment variable need to be set, or use cygnss_l1_path input parameter")

    cyg_folder = os.path.join(cygnss_l1_path, '{:04d}'.format(data_year), '{:03d}'.format(data_day))

    result = list()
    pattern = "cyg{:02d}*.nc".format(sc_num)
    for root, dirs, files in os.walk(cyg_folder):
        for name in files:
            if fnmatch.fnmatch(name, pattern):
                result.append(name)
    if not result:
        raise FileNotFoundError("No file for sc: {:d} found in {:s}".format(sc_num, cyg_folder))

    cyg_file_name = result[0]
    return cyg_file_name


def get_sample_id_from_time_rltv_tcs(cyg_full_file, sel_sample_time):
    """
    get sample id (zero-based) from the time (in seconds) of the selected DDM (sample) relative to the time_coverage_start
    :param cyg_full_file: cygnss file
    :type cyg_full_file: str
    :param sel_sample_time: time (in seconds) of the selected DDM (sample) relative to the time_coverage_start
    :type sel_sample_time: float
    :return: L1 sample id (zero-based)
    :type: int
    """
    ds_cyg = Dataset(cyg_full_file, 'r')
    ds_cyg.set_auto_maskandscale(False)

    tcs = pd.Timestamp(ds_cyg.time_coverage_start)
    tcs_in_sec = (tcs - copy.deepcopy(tcs).replace(hour=0, minute=0, second=0, microsecond=0, nanosecond=0)).delta * 1e-9

    sample_time = np.array(ds_cyg.variables['ddm_timestamp_utc']) + tcs_in_sec  # Number of seconds relative to the beginning of the day [sec]
    # resolution is nano second
    ds_cyg.close()

    sel_ddm_tf = np.isclose(sample_time, sel_sample_time, rtol=0, atol=1e-6)  # set atol=1e-9 as the resolution in nano second
    if sel_ddm_tf.sum() > 1:  # if we have two samples with the same time tamp
        raise RuntimeError("Detecting two samples with time sample less than 1 nS.")
    if sel_ddm_tf.sum() == 0:
        warnings.warn('No sample with time relative to tcs {:f} s, in {:s}'.format(sel_sample_time, cyg_full_file))
        l1_sample_id = np.nan
    else:
        l1_sample_id = int(np.where(sel_ddm_tf)[0])
    return l1_sample_id


def uppath(path, n):
    path_split = path.split(os.sep)
    if path_split[-1] == '':
        n += 1
    return os.sep.join(path.split(os.sep)[:-n])


def cyg_id_list_from_kml(in_kml):
    """
    create list of CygDdmId from kml file

    :param str in_kml: kml file
    :return: list of CygDdmId
    :rtype: list of CygDdmId
    """
    ddms_list = get_list_ddm_info_from_kml(in_kml)
    cyg_ddm_id_list = list()
    for ddm_info in ddms_list:
        cyg_ddm_id_list.append(CygDdmId(None, ddm_info['year'], ddm_info['day'], ddm_info['sc_num'], ddm_info['ch_id'], ddm_info['samp_id'],
                                        sample_time_sec=ddm_info['time_rltv_tsc_ns'], ddm_tag=ddm_info['tag']))

    return cyg_ddm_id_list


class CygDdmId:

    def __init__(self, file_name, year, day, sc_num, ch_id, samp_id, land_samp_id=None, sample_time_sec=None, land_file_name=None, ddm_tag=None):
        """
        This class hold the info needed to select specific CYGNSS DDM

        :param file_name: file name
        :type file_name: str or None
        :param year: data year, ex. 2019
        :type year: int
        :param day: day of the year, (1-36X)
        :type day: int
        :param sc_num: spacecraft id, (1-8)
        :type sc_num: int
        :param ch_id: channel id (1-4) (one-based)
        :type ch_id: int
        :param samp_id: sample id (zero-based)
        :type samp_id: int
        :param land_samp_id: sample id of the land product (zero-based)
        :type land_samp_id: int or None
        :param sample_time_sec: time (in seconds) of the selected DDM (sample) from the beginning of the day
        :type sample_time_sec: float or None
        :param land_file_name: land product file name
        :type land_file_name: str or None
        """
        if land_samp_id is not None:
            land_samp_id = int(land_samp_id)

        self.file_name = file_name

        self.year = int(year)
        self.day = int(day)
        self.ch_id = int(ch_id)
        self.samp_id = int(samp_id)
        self.sc_num = int(sc_num)
        self.land_samp_id = land_samp_id
        self.sample_time_sec = sample_time_sec  # easy way of identifying sample ID for the different products
        self.land_file_name = land_file_name  # land product file name
        self.ddm_tag = ddm_tag

    def set_land_sample_id(self, land_samp_id):
        """
        set land_samp_id field

        :param land_samp_id: sample id of the land product (zero-based)
        :type land_samp_id: int
        :return:
        """
        self.land_samp_id = int(land_samp_id)

    def set_ddm_time(self, sample_time_sec):
        """
        set sample_time_sec field

        :param sample_time_sec: time (in seconds) of the selected DDM (sample) from the beginning of the day
        :type sample_time_sec: float or None
        :return:
        """
        self.sample_time_sec = sample_time_sec

    def set_land_file_name(self, land_file_name):
        """
        set land_file_name field

        :param land_file_name: land product file name
        :type land_file_name: str
        :return:
        """
        self.land_file_name = land_file_name

    def fill_file_name(self, cygnss_l1_path=os.environ.get("CYGNSS_L1_PATH")):
        """
        search for the file name and fill the file_name name
        :param cygnss_l1_path: path of the cygnss L1 data (default: os.environ.get("CYGNSS_L1_PATH"))
        :type cygnss_l1_path: str or None
        :return:
        """
        self.file_name = find_cygnss_file(data_year=self.year, data_day=self.day, sc_num=self.sc_num, cygnss_l1_path=cygnss_l1_path)

    def fill_land_parameters(self, cygnss_l1_path=os.environ.get("CYGNSS_L1_PATH"), l1_ocean_folder_name=None, l1_land_folder_name=None):
        """
        using samp_id or sample_time_sec as the sample id for the ocean product, this function fill land_samp_id, sample_time_sec,
        land_file_name fields.

        Note only the fields with None value will be filled.

        :param cygnss_l1_path: path of the cygnss L1 data (default: os.environ.get("CYGNSS_L1_PATH"))
        :type cygnss_l1_path: str or None
        :param l1_ocean_folder_name: L1 ocean folder name (default: L1_OCEAN_FOLDER)
        :type l1_ocean_folder_name: str or None
        :param l1_land_folder_name: L1 land folder name (default: L1_LAND_FOLDER)
        :type l1_land_folder_name: str or None
        :return:
        """
        if cygnss_l1_path is None:
            cygnss_l1_path = os.environ.get("CYGNSS_L1_PATH")
            if cygnss_l1_path is None:
                raise ValueError("$CYGNSS_L1_PATH environment variable need to be set, or use cygnss_l1_path input parameter")

        l1_land_folder_name = L1_LAND_FOLDER if l1_land_folder_name is None else l1_land_folder_name
        l1_ocean_folder_name = L1_OCEAN_FOLDER if l1_ocean_folder_name is None else l1_ocean_folder_name
        cygnss_l1_root_path = uppath(cygnss_l1_path, 1)
        cygnss_l1_land_path = os.path.join(cygnss_l1_root_path, l1_land_folder_name)

        if self.land_file_name is None:
            self.land_file_name = find_cygnss_file(data_year=self.year, data_day=self.day, sc_num=self.sc_num, cygnss_l1_path=cygnss_l1_land_path)

        if self.sample_time_sec is not None:
            cygnss_land_file = os.path.join(cygnss_l1_land_path, '{:04d}'.format(self.year), '{:03d}'.format(self.day), self.land_file_name)
            self.land_samp_id = get_sample_id_from_time_rltv_tcs(cyg_full_file=cygnss_land_file, sel_sample_time=self.sample_time_sec)
        else:
            self.land_file_name, self.land_samp_id, self.sample_time_sec = get_land_prod_info_from_ocean_prod(
                self.year, self.day, self.sc_num, self.samp_id, cygnss_l1_path=cygnss_l1_path, l1_ocean_folder_name=l1_ocean_folder_name,
                l1_land_folder_name=l1_land_folder_name)

    def get_utc_time(self):
        """
        Return DDM sample time

        :return: DDM UTC Time
        :rtype: pd.Timestamp
        """
        day_date = pd.Timestamp(self.year, 1, 1) + pd.Timedelta(days=self.day-1)
        ddm_time = day_date + pd.Timedelta(int(self.sample_time_sec * 1e9), 'ns')

        return ddm_time



#!/usr/bin/env python3
"""
 DESCRIPTION
          This tool is part of cygnsslib python package. The package is created by Mixil lab at USC
          See <https://bitbucket.org/usc_mixil/cygnss-library>

          This Tool download SRTM .hgt data

 AUTHOR   Amer Melebari
          Microwave Systems, Sensors and Imaging Lab (MiXiL)
          University of Southern California
 EMAIL    jamesdca@usc.edu
 CREATED  2020-08-16
 Updated  2020-06-22


part of the download_file() is from the following codes:
https://git.earthdata.nasa.gov/projects/LPDUR/repos/daac_data_download_python/
https://github.com/DHI-GRAS/earthdata-download/tree/master/earthdata_download

  Copyright 2020 University of Southern California
"""
from cygnsslib.data_downloader.pass_core import MixilKeys
from getpass import getpass
from time import sleep
from tqdm.auto import tqdm
import numpy as np
import os
import requests
import shutil
import zipfile

CHUNK_SIZE = 512 * 1024  # 512 kB
MAX_CONNECTIONS_TRIES = 10
MIN_FILE_SIZE_BYTES = 10e3
SRTM_USGS_URL = 'https://e4ftl01.cr.usgs.gov/MEASURES/SRTMGL1.003/2000.02.11/'
"""
"""


class EarthdataSession(requests.Session):

    AUTH_DOMAINS = ['nasa.gov', 'usgs.gov']

    def __init__(self, username, password):
        """Create Earthdata Session that preserves headers when redirecting"""
        super(EarthdataSession, self).__init__()  # Python 2 and 3
        self.auth = (username, password)

    def rebuild_auth(self, prepared_request, response):
        """Keep headers upon redirect as long as we are on any of self.AUTH_DOMAINS"""
        headers = prepared_request.headers
        url = prepared_request.url
        if 'Authorization' in headers:
            original_parsed = requests.utils.urlparse(response.request.url)
            redirect_parsed = requests.utils.urlparse(url)
            original_domain = '.'.join(original_parsed.hostname.split('.')[-2:])
            redirect_domain = '.'.join(redirect_parsed.hostname.split('.')[-2:])
            if original_domain != redirect_domain and redirect_domain not in self.AUTH_DOMAINS and original_domain not in self.AUTH_DOMAINS:
                del headers['Authorization']


def _cyg_data_auth_err_handling():
    print('Error in the Earthdata Credentials, try to enter them again.')
    print('Note: you need to re-start the code after this download as the new user/pass need to be re-loaded')
    save_pass_str = input('Do you want to save the new user/pass? (if yes, the old user/pass will be deleted) [Yes, No]')
    save_pass = True if (save_pass_str.lower() in ['y', 'yes']) else False
    podaac_usr, podaac_pass = get_earthdata_cred(pass_folder=None, save_pass=save_pass)
    return podaac_usr, podaac_pass


def _download_file_earthdata(file_url, output_folder, auth):
    """
    download the file with url into folder output_folder

    :param file_url: url of the file
    :type file_url: str
    :param output_folder: saving folder
    :type output_folder: str
    :param auth: username and pass
    :type auth: tuple of str
    :return: downloaded file path
    :rtype: str
    """
    file_name = file_url.split('/')[-1]
    out_file = os.path.join(output_folder, file_name)

    out_file_temp = f'{out_file:s}.incomplete'
    for i_try in np.arange(MAX_CONNECTIONS_TRIES):
        try:
            with EarthdataSession(username=auth[0], password=auth[1]) as session:
                with session.get(file_url, stream=True) as response:
                    response.raise_for_status()
                    response.raw.decode_content = True
                    with tqdm.wrapattr(open(out_file_temp, "wb"), "write", miniters=1, total=int(response.headers.get('content-length', 0)),
                                       desc=out_file_temp) as target_file:
                        for chunk in response.iter_content(chunk_size=CHUNK_SIZE):
                            target_file.write(chunk)

            filesize = os.path.getsize(out_file_temp)
            if filesize < MIN_FILE_SIZE_BYTES:
                raise RuntimeError(f'Size of downloaded file is only {(filesize * 1e-3):.3f} B. Suspecting broken file ({out_file_temp}).')

        except (requests.exceptions.ConnectionError, requests.exceptions.HTTPError) as e:
            if i_try < MAX_CONNECTIONS_TRIES:
                print('Connection Error sleep for 5 sec')
                sleep(5)
            else:
                raise RuntimeError(f'a connection error was raised when trying to access {file_url:s}\n {e}')
        except requests.exceptions.RequestException:
            if i_try < MAX_CONNECTIONS_TRIES:
                podaac_usr, podaac_pass = _cyg_data_auth_err_handling()
                auth = (podaac_usr, podaac_pass)
            else:
                raise RuntimeError('Authentication error, please check your PODAAC user/pass')
        except RuntimeError as e:
            if i_try < MAX_CONNECTIONS_TRIES:
                continue
            else:
                raise RuntimeError(e)

        else:  # if it was successful
            break

    shutil.move(out_file_temp, out_file)

    return out_file


def extract_srtm_zipfile(zip_file, out_path):
    """
    unzip file

    :param zip_file: zip file path
    :type zip_file: str
    :param out_path: extraction out folder
    :type out_path: str
    :return: unzipped file name
    :rtype: str
    """

    with zipfile.ZipFile(zip_file, "r") as zip_ref:
        file_name = zip_ref.namelist()[0]
        zip_ref.extractall(out_path, members=[file_name])

    return os.path.join(out_path, file_name)


def get_srtm_file_name(lat, lon):
    """
    get SRTM file name. the name of the zipped file, for the unzipped file use get_srtm_ght_file_name_frm_lat_lon() or get_srtm_unzipped_file_name()
    :param lat: latitude
    :type lat: int or float
    :param lon: longitude
    :type lon: int or float
    :return:
    """
    file_coord = _get_srtm_file_coord_name(lat, lon)
    file_name = f'{file_coord:s}.SRTMGL1.hgt.zip'
    return file_name


def _get_srtm_file_coord_name(lat, lon):
    """

    :param lat: latitude
    :type lat: int or float
    :param lon: longitude
    :type lon: int or float
    :return:
    """
    lat = int(lat)
    lon = int(lon)
    if lat > 59 or lat < -56:
        raise ValueError('Latitude value should be between -56 to 59. Got {:d}'.format(lat))
    if lon > 179 or lon < -180:
        raise ValueError('Longitude value should be between -180 to 179. Got {:d}'.format(lon))
    lat_direction = 'N' if (lat >= 0) else 'S'
    lon_direction = 'E' if (lon >= 0) else 'W'
    file_coord = '{:s}{:02d}{:s}{:03d}'.format(lat_direction, abs(lat), lon_direction, abs(lon))
    return file_coord


def get_srtm_ght_file_name(lat, lon):
    """
    return the name of the .hgt file from lat, lon value
    :param lat: latitude
    :type lat: int or float
    :param lon: longitude
    :type lon: int or float
    :return:
    """
    file_coord = _get_srtm_file_coord_name(lat, lon)
    file_name = f'{file_coord:s}.hgt'
    return file_name


def get_srtm_unzipped_file_name(zipped_filename):
    """
    return the unzipped SRTM file name from the zipped SRTM file name

    :param zipped_filename: zipped SRTM file name from get_srtm_file_name()
    :type zipped_filename: str
    :return: unzipped SRTM file name
    :rtype: str
    """
    return '{:s}.hgt'.format(zipped_filename.split('.')[0])


def get_earthdata_cred(pass_folder=None, save_pass=True, reset_pass=False):
    """
    import EarthData username and pass from the system, if not found it ask you to enter them

    :return:
    """
    if reset_pass:
        mixil_keys = MixilKeys(pass_folder=pass_folder)
        mixil_keys.remove('lpdaac_usr')
        mixil_keys.remove('lpdaac_pass')
        del mixil_keys

    if save_pass:
        mixil_keys = MixilKeys(pass_folder=pass_folder)
        mixil_keys.require('lpdaac_usr', msg='Get NASA Earthdata Credentials from: https://urs.earthdata.nasa.gov/home \nNASA Earthdata Username: ')
        mixil_keys.require('lpdaac_pass', msg='NASA Earthdata Password: ')
        earthdata_usr = mixil_keys.retrieve('lpdaac_usr')
        earthdata_pass = mixil_keys.retrieve('lpdaac_pass')
    else:
        earthdata_usr = getpass('NASA Earthdata Username: ')
        earthdata_pass = getpass('NASA Earthdata Password: ')

    return earthdata_usr, earthdata_pass


def download_srtm_ght_files(list_lat, list_lon, out_folder=None, earthdata_usr_pass=None, save_pass=True, unzipfile=False, remove_zippedfile=True,
                            silent_run=False):
    """
    download a multiple SRTM file in ght format. it download the files within the lat/long lists

    The files are in this URL: https://e4ftl01.cr.usgs.gov/MEASURES/SRTMGL1.003/2000.02.11/
    the username/password can be obtained from https://urs.earthdata.nasa.gov/home

    :param list_lat: list of latitude
    :type list_lat: list of int or numpy.array or int
    :param list_lon: list of longitude
    :type list_lon: list of int or or numpy.array or int
    :param out_folder: save folder, if None, it will be saved in "$SRTM_PATH/hgt"
    :type out_folder: str
    :param earthdata_usr_pass: earthdata username and password (usr,pass).  https://urs.earthdata.nasa.gov/home
    :type earthdata_usr_pass: tuple of str
    :param save_pass: (active if earthdata_usr_pass is None) save earthdata username and password in the system (select False if you're using a shared
     or public PC)
    :type save_pass: bool
    :param unzipfile: unzip the downloaded file?
    :type unzipfile: bool
    :param remove_zippedfile: delete the unzipped file after unzipping?
    :type remove_zippedfile: bool
    :param silent_run: if true no screen output will be shown except downloading progress and errors
    :type silent_run: bool
    :return: files path
    :rtype: list of str
    """
    try:
        iter(list_lat)
    except TypeError:
        list_lat = [int(list_lat)]
    try:
        iter(list_lon)
    except TypeError:
        list_lon = [int(list_lon)]

    if out_folder is None:
        out_folder = os.path.join(os.environ['SRTM_PATH'], 'hgt')
        os.makedirs(out_folder, exist_ok=True)
    if earthdata_usr_pass is None:
        earthdata_usr_pass = get_earthdata_cred(save_pass=save_pass)

    d_file_paths = list()
    for lat in list_lat:
        for lon in list_lon:
            d_file_paths.append(download_srtm_ght_single_file(lat, lon, out_folder=out_folder, earthdata_usr_pass=earthdata_usr_pass,
                                                              save_pass=save_pass, unzipfile=unzipfile, remove_zipedfile=remove_zippedfile,
                                                              silent_run=silent_run))

    return d_file_paths


def download_srtm_ght_single_file(lat, lon, out_folder=None, earthdata_usr_pass=None, save_pass=True, unzipfile=False, remove_zipedfile=True,
                                  silent_run=False):
    """

    download a single SRTM file in ght format, don't use this function directly, consider using download_srtm_ght_files()
    The files are in this URL: https://e4ftl01.cr.usgs.gov/MEASURES/SRTMGL1.003/2000.02.11/
    the username/password can be obtain from https://urs.earthdata.nasa.gov/home

    :param lat: file latitude
    :type lat: int
    :param lon: file longitude
    :type lon: int
    :param out_folder: save folder, if None, it will be saved in "$SRTM_PATH/hgt"
    :type out_folder: str
    :param earthdata_usr_pass: earthdata username and password (usr,pass).  https://urs.earthdata.nasa.gov/home
    :type earthdata_usr_pass: tuple of string
    :param save_pass: (active if earthdata_usr_pass is None) save earthdata username and password in the system (select False if you're using a shared
     or public PC)
    :type save_pass: bool
    :param unzipfile: unzip the downloaded file?
    :type unzipfile: bool
    :param remove_zipedfile: delete the unzipped file after unzipping? ignored if unzipfile is False
    :type remove_zipedfile: bool
    :param silent_run: if true no screen output will be shown except downloading progress
    :type silent_run: bool
    :return: file path
    :rtype: str
    """
    if out_folder is None:
        out_folder = os.path.join(os.environ['SRTM_PATH'], 'hgt')
        os.makedirs(out_folder, exist_ok=True)

    file_name = get_srtm_file_name(lat, lon)
    file_path = os.path.join(out_folder, file_name)
    unzipped_file_path = os.path.join(out_folder, get_srtm_unzipped_file_name(file_name))
    if os.path.exists(unzipped_file_path):
        if not silent_run:
            print(f'{unzipped_file_path:s} file exist')
        return unzipped_file_path
    if os.path.exists(file_path):
        if not silent_run:
            print(f'{file_path:s} file exist')
        return file_path

    if earthdata_usr_pass is None:
        earthdata_usr_pass = get_earthdata_cred(save_pass=save_pass)
    file_url = f'{SRTM_USGS_URL:s}{file_name:s}'
    saved_zip_file = _download_file_earthdata(file_url, out_folder, auth=earthdata_usr_pass)
    if not unzipfile:
        return saved_zip_file

    extract_srtm_zipfile(saved_zip_file, out_folder)
    unzipped_file = os.path.join(out_folder, get_srtm_unzipped_file_name(file_name))
    if remove_zipedfile:
        os.remove(saved_zip_file)
    return unzipped_file


if __name__ == '__main__':
    files_lat = np.array([21, 22, 20])
    files_lon = [29, 30]
    download_srtm_ght_files(files_lat, files_lon)




import os
from cygnsslib import create_centered_polygon, write_sp_within_radius

if __name__ == '__main__':
    # CYGNSS_L1_dir = os.path.join("G:\\", "cygnss_data", "L1", "v3")
    CYGNSS_L1_dir = None
    year = 2020
    daylist = list(range(1, 153))

    radius = 10e3
    thesh_ddm_snr = 3
    thesh_noise = 1
    out_root = "z1_2020_v3"
    # ref_pos = [37.1898, -105.9903] # Z4
    ref_pos = [37.1906, -105.9921]  # Z1
    write_sp_within_radius(CYGNSS_L1_dir, year, daylist, ref_pos, radius, out_root, thesh_ddm_snr, thesh_noise)
    create_centered_polygon(ref_pos, radius, "a", shape="circle")



from Crypto.Cipher import AES
from getpass import getpass
from pbkdf2 import PBKDF2
import base64
import os
import pickle

# Settings ###
try:
    saltSeed = os.environ['MIXIL_KEYS_SEED']
except KeyError:
    saltSeed = 'VR7g]^"^=S/76ZBU;g,'  # MAKE THIS YOUR OWN RANDOM STRING

PASSPHRASE_SIZE = 64  # 512-bit passphrase
KEY_SIZE = 32  # 256-bit key
BLOCK_SIZE = 16  # 16-bit blocks
IV_SIZE = 16  # 128-bits to initialise
SALT_SIZE = 8  # 64-bits of salt


def get_salt_for_key(key):
    return PBKDF2(key, saltSeed).read(SALT_SIZE)  # Salt is generated as the hash of the key with it's own salt acting like a seed value


def clear_all_keys(pass_folder=None):
    """
    Delete the files that have the keys. aka clear all the keys

    :param pass_folder: the folder where the encrypted files are stored, if None, will use $HOME
    :type pass_folder: str or None
    :return: void
    """
    if pass_folder is None:
        pass_folder = os.getenv("HOME")

    pass_phrase_file = os.path.join(pass_folder, '.secret.p')
    secret_sdb_file = os.path.join(pass_folder, '.secrets')

    if os.path.exists(pass_phrase_file):
        os.remove(pass_phrase_file)
    if os.path.exists(secret_sdb_file):
        os.remove(secret_sdb_file)


class MixilKeys:
    """
    This Class Store/retrieve usernames/passwords.
    by default it store the keys in the following files: $home/.secret.p and $home/.secrets

    you can make it more secure by setting your own Seed for the encryption, this can be done by setting the environment variable $Mixil_keys_seed


    *The code is based on the code in
    https://stackoverflow.com/questions/7014953/i-need-to-securely-store-a-username-and-password-in-python-what-are-my-options

    """

    def __init__(self, pass_folder=None):
        """

        you can make it more secure by setting your own Seed for the encryption, this can be done by setting the environment variable $Mixil_keys_seed

        :param pass_folder: the folder where the encrypted files are stored, if None, will use $HOME
        :type pass_folder: str or None
        """
        # Setup
        if pass_folder is None:
            pass_folder = os.getenv("HOME")

        self.pass_phrase_file = os.path.join(pass_folder, '.secret.p')
        self.secret_sdb_file = os.path.join(pass_folder, '.secrets')
        if os.path.isdir(self.pass_phrase_file) or os.path.isdir(self.secret_sdb_file):
            raise NameError('There is a folder with the same name, you need to remove the folders: {:s} and {:s} '
                            .format(self.pass_phrase_file, self.secret_sdb_file))
        # Acquire passphrase:
        try:
            with open(self.pass_phrase_file, 'rb') as f:
                passphrase = f.read()
            if len(passphrase) == 0:
                raise IOError
        except IOError:
            with open(self.pass_phrase_file, 'wb') as f:
                passphrase = os.urandom(PASSPHRASE_SIZE)  # Random passphrase
                f.write(base64.b64encode(passphrase))

                try:
                    os.remove(self.secret_sdb_file)  # If the passphrase has to be regenerated, then the old secrets file is irretrievable
                    # and should be removed
                except:
                    pass
        # else:
        #     passphrase = base64.b64decode(passphrase)  # Decode if loaded from already extant file

        # Load or create secrets database:
        try:
            with open(self.secret_sdb_file, 'rb') as f:
                db = pickle.load(f)
            if db == {}:
                raise IOError
        except (IOError, EOFError):
            db = {}
            with open(self.secret_sdb_file, 'wb') as f:
                pickle.dump(db, f)

    # System Functions

    def _read_passphrase(self):
        with open(self.pass_phrase_file, 'rb') as f:
            passphrase = f.read()
            passphrase = base64.b64decode(passphrase)
            return passphrase

    def _get_db(self):

        with open(self.secret_sdb_file, 'rb') as f:
            db = pickle.load(f)
        return db

    def encrypt(self, plaintext, salt):
        """
        Pad plaintext, then encrypt it with a new, randomly initialised cipher. Will not preserve trailing whitespace in plaintext!
        """

        # Initialise Cipher Randomly
        init_vector = os.urandom(IV_SIZE)

        # Prepare cipher key:
        key = PBKDF2(self._read_passphrase(), salt).read(KEY_SIZE)

        cipher = AES.new(key, AES.MODE_CBC, init_vector)  # Create cipher

        return init_vector + cipher.encrypt(plaintext + ' ' * (BLOCK_SIZE - (len(plaintext) % BLOCK_SIZE)))  # Pad and encrypt

    def decrypt(self, ciphertext, salt):
        """ Reconstruct the cipher object and decrypt. Will not preserve trailing whitespace in the retrieved value!"""

        # Prepare cipher key:
        key = PBKDF2(self._read_passphrase(), salt).read(KEY_SIZE)

        # Extract IV:
        init_vector = ciphertext[:IV_SIZE]
        ciphertext = ciphertext[IV_SIZE:]

        cipher = AES.new(key, AES.MODE_CBC, init_vector)  # Reconstruct cipher (IV isn't needed for decryption so is set to zeros)

        return cipher.decrypt(ciphertext).rstrip(b' ')  # Decrypt and de-pad

    # User Functions ###

    def store(self, key, value):

        """
        Store key-value pair safely and save to disk.

        :param key: key name
        :type key: str
        :param value: key value
        :type value: str

        """
        db = self._get_db()

        db[key] = self.encrypt(value, get_salt_for_key(key))
        with open(self.secret_sdb_file, 'wb') as f:
            pickle.dump(db, f)

    def retrieve(self, key):
        """ Fetch key-value pair."""
        db = self._get_db()
        return self.decrypt(db[key], get_salt_for_key(key)).decode("utf-8")

    def require(self, key, msg=None):
        """
        Test if key is stored, if not, prompt the user for it while hiding their input from shoulder-surfers.

        :param key: key name
        :type key: str
        :param msg: message for entering the key, if None it uses a default message
        :type msg: str or None
        """
        keys_db = self._get_db()
        if key not in keys_db:
            if msg is None:
                msg = 'Please enter a value for "{:s}":'.format(key)
            self.store(key, getpass(msg))

    def remove(self, key):
        """
        Remove the key if exist
        :param key: key name
        :type key: str
        :return: True if it was removed, False else
        """
        db = self._get_db()
        if key in self._get_db():
            db.pop(key, None)
            with open(self.secret_sdb_file, 'wb') as f:
                pickle.dump(db, f)
            return True

        return False


if __name__ == '__main__':
    # Test (put your code here) ###
    mixil_keys = MixilKeys()

    mixil_keys.require('id')
    mixil_keys.require('password1')
    mixil_keys.require('password2')
    print('')
    print('Stored Data:')

    db_temp = mixil_keys._get_db()
    for key in db_temp:
        print('{:s}: {:s}'.format(key, mixil_keys.retrieve(key)))  # decode values on demand to avoid exposing the whole database in memory



from cygnsslib.data_downloader.download_cygnss import download_cyg_files, download_cyg_files_between_date, get_podaac_cred, get_cyg_file
from cygnsslib.data_downloader.download_cyg_rawif import download_rawif_cyg_files_between_date, download_cyg_rawif_files, get_cyg_rawif_files



from cygnsslib.data_downloader.download_cygnss import download_file
import os
import shutil
import zipfile

ANT_PTRN_URL = 'https://bitbucket.org/usc_mixil/cygnss_antenna_patterns/get/master.zip'


def download_cyg_antenna_patterns(antenna_patterns_folder_path=None):
    if antenna_patterns_folder_path is None:
        antenna_patterns_folder_path = os.path.join(os.environ["CYGNSS_PATH"], 'cygnss_antenna_patterns')
    if not os.path.isdir(antenna_patterns_folder_path):
        os.makedirs(antenna_patterns_folder_path)

    files_list = os.listdir(antenna_patterns_folder_path)
    if 'antennaRx_CYGNSS_Obs1_Nadir02_Starboard_processed' in files_list and 'antennaRx_CYGNSS_Obs1_Nadir01_Port_processed' in files_list:
        ant_ptrn_files_path = [os.path.join(antenna_patterns_folder_path, 'antennaRx_CYGNSS_Obs1_Nadir02_Starboard_processed'),
                               os.path.join(antenna_patterns_folder_path, 'antennaRx_CYGNSS_Obs1_Nadir01_Port_processed')]
        return ant_ptrn_files_path

    ant_ptrn_zip_path = os.path.join(antenna_patterns_folder_path, ANT_PTRN_URL.split('/')[-1])
    if not os.path.exists(ant_ptrn_zip_path):
        ant_ptrn_zip_path = download_file(ANT_PTRN_URL, antenna_patterns_folder_path, auth=None)

    ant_ptrn_files_path = extract_ant_ptrn_zip(ant_ptrn_zip_path, antenna_patterns_folder_path)
    os.remove(ant_ptrn_zip_path)  # remove zip file
    return ant_ptrn_files_path


def extract_ant_ptrn_zip(zip_file_path, out_path):

    with zipfile.ZipFile(zip_file_path, "r") as zip_ref:

        files_list = zip_ref.namelist()
        extracted_files = list()
        for file_name in files_list:
            if file_name.endswith('processed'):
                zip_ref.extract(file_name, path=out_path)
                shutil.move(os.path.join(out_path, file_name), os.path.join(out_path, file_name.split(os.sep)[-1]))
                extracted_files.append(os.path.join(out_path, file_name.split(os.sep)[-1]))

    os.rmdir(os.path.join(out_path, file_name.split(os.sep)[0]))
    return extracted_files


if __name__ == '__main__':
    download_cyg_antenna_patterns()



#!/usr/bin/env python3

#  FILE     TBD
#  DESCRIPTION
#           TBD
#  AUTHOR   James D. Campbell
#           Microwave Systems, Sensors and Imaging Lab (MiXiL)
#           University of Southern California
#  EMAIL    jamesdca@usc.edu
#  CREATED  2018-12-12
#
#  Copyright 2018 University of Southern California

import cygnsslib
import os
import numpy as np

if __name__ == '__main__':
    cygnss_l1_path = os.environ["CYGNSS_L1_PATH"]
    year = 2019

    daylist = np.arange(1, 30)

    radius = 5e3
    thesh_ddm_snr = 3
    thesh_noise = 1
    out_root = 'yanco_2019'
    run_options = {'img_save_type': ['png', 'eps'], 'save_cvs': True, 'save_ddm_img': False, 'sheet_type': 'xlsx', 'multithreads': True,
                   'silent_mode': True}
    ref_pos = [-34.856660, 146.140752]  # Yonco
    out_high, out_low, out_ref = cygnsslib.write_sp_within_radius(cygnss_l1_path, year, daylist, ref_pos, radius, out_root, thesh_ddm_snr,
                                                                  thesh_noise, out_options=run_options)

    out_kml = f'grp1_{out_high:s}'
    cygnsslib.group_sp_within_distance(out_high, out_kml=out_kml, max_dist=100, save_csv=True)



import os

import cygnsslib.data_downloader.download_cyg_rawif

try:
    from cygnsslib import cygnss_download
except ImportError:
    import sys
    parent_dir_path = os.path.abspath(os.path.join(os.path.dirname(os.path.realpath(__file__)), os.pardir))
    sys.path.insert(0, parent_dir_path)
    from cygnsslib import cygnss_download
import numpy as np
import datetime as dt

if __name__ == '__main__':

    # data_day = np.arange(5, 10)
    data_year = 2020
    # sc_num = 3
    # sc_num = None
    check_md5_exist_file = False
    cyg_data_ver = 'v2.1'
    cygnss_l1_path = os.environ["CYGNSS_L1_PATH"]
    data_day = 24
    sc_num = 3
    cygnss_download.download_cyg_files(data_year, data_day, list_sc_num=sc_num, cyg_data_ver=cyg_data_ver, cyg_data_lvl='L1',
                                       cygnss_l1_path=cygnss_l1_path, check_md5_exist_file=check_md5_exist_file)
    st_date = dt.date(year=2019, month=1, day=12)
    end_date = dt.date(year=2020, month=1, day=3)

    cygnss_download.download_cyg_files_between_date(st_date, end_date, list_sc_num=sc_num, cyg_data_ver=cyg_data_ver, cyg_data_lvl='L1',
                                                    cygnss_l1_path=cygnss_l1_path, check_md5_exist_file=check_md5_exist_file)

    cygnsslib.data_downloader.download_cyg_rawif.download_cyg_rawif_files(data_year=2020, list_data_day=227)

    st_date = dt.date(year=2020, month=8, day=4)
    end_date = dt.date(year=2020, month=8, day=4)
    cygnsslib.data_downloader.download_cyg_rawif.download_rawif_cyg_files_between_date(st_date, end_date, download_l1_data=True)




try:
    from xlutils.copy import copy as sheet_copy
    import xlrd
except ImportError:
    pass
import numpy as np
import os

"""
NOTE: This is only left for compatibility with old versions, no update will be done to this code. 
use Pandas instead of this 
"""


def convert_data(value):
    if type(value) == np.ndarray:
        value = value.tolist()
    if type(value) in (complex, np.complex_, np.complex128, np.complex64):
        value = complex(value)
    if type(value) in (np.float_, np.float32):
        value = float(value)
    if type(value) in (np.int64, np.int32, np.int_, np.int8):
        value = int(value)
    if type(value) in (bool, np.bool_):
        value = 1 if value else 0
    return value


def write_row_existing_xls(xls_file_path, data, row_idx=-1, col_offset=0, sheet_id=0):
    """
    write a row in an existing xls file

    :param xls_file_path: xls file path
    :type xls_file_path: str
    :param data: data in an np.array or list
    :type data: list or np.array
    :param row_idx: row index, select -1 for adding a new row
    :type row_idx: int
    :param col_offset: number of column offset
    :type col_offset: int
    :param sheet_id: sheet index, default 0 (the first one)
    :type sheet_id: int
    :return: void
    """
    if not os.path.isfile(xls_file_path):
        raise FileExistsError(f'xls file not exist in path {xls_file_path}')
    rb = xlrd.open_workbook(xls_file_path, formatting_info=True)
    r_sheet = rb.sheet_by_index(sheet_id)
    row = r_sheet.nrows
    wb = sheet_copy(rb)
    sheet = wb.get_sheet(0)
    if row_idx == -1:
        row_idx = row
    write_xls_row(sheet, data, row_idx, col_offset=col_offset)
    wb.save(xls_file_path)


def write_xls_row(sheet, data, row, col_offset=0):
    """
    write a row in an xls sheet

    :param sheet: sheet object
    :type sheet: xlrd.Sheet
    :param data: data in an np.array or list
    :type data: list or np.array
    :param row: row index
    :type row: int
    :param col_offset: number of column offset
    :type col_offset: int
    :return: void
    """

    for col, col_val in enumerate(data):
        sheet.write(row, col + col_offset, convert_data(col_val))


def find_col_header(text, header):
    """

    find the col idx from the header, if multiple idx found it'll return a list, if none was found it'll return None

    :param text: header text
    :type text: str
    :param header: header list
    :type header: list
    :return: col idx
    :rtype int or None
    """

    out_idx = list()
    for idx, value in enumerate(header):
        if text == value:
            out_idx.append(idx)
    if len(out_idx) < 1:
        out_idx = None
    elif len(out_idx) < 2:
        out_idx = out_idx[0]
    return out_idx


def write_col_existing_xls(xls_file_path, data, col_idx=-1, row_offset=0, sheet_id=0):
    """
    write a row in an existing xls file

    :param xls_file_path: xls file path
    :type xls_file_path: str
    :param data: data in an np.array or list
    :type data: list or np.array
    :param col_idx: column index, select -1 for adding a new column
    :type col_idx: int
    :param row_offset: number of row offset
    :type row_offset: int
    :param sheet_id: sheet index, default 0 (the first one)
    :type sheet_id: int
    :return: void
    """
    if not os.path.isfile(xls_file_path):
        raise FileExistsError(f'xls file not exist in path {xls_file_path}')

    rb = xlrd.open_workbook(xls_file_path, formatting_info=True)
    r_sheet = rb.sheet_by_index(sheet_id)
    col = r_sheet.ncols
    wb = sheet_copy(rb)
    sheet = wb.get_sheet(0)
    if col_idx == -1:
        col_idx = col
    write_xls_col(sheet, data, col_idx, row_offset=row_offset)
    wb.save(xls_file_path)


def write_xls_col(sheet, data, col, row_offset=0):
    """
    write a column in an xls sheet

    :param sheet: sheet object
    :type sheet: xlrd.Sheet
    :param data: data in an np.array or list
    :type data: list or np.array
    :param col: column index
    :type col: int
    :param row_offset: number of row offset
    :type row_offset: int
    :return: void
    """

    for row, row_val in enumerate(data):
        sheet.write(row + row_offset, col, convert_data(row_val))



import datetime as dt
import fnmatch
import numpy as np
import os
import warnings

from cygnsslib.data_downloader.download_cygnss import PODAAC_CYG_URL, _get_cyg_page_data, download_file, download_cyg_files, get_podaac_cred


def download_cyg_rawif(data_year, data_day, sc_num, out_folder, podaac_usr_pass, day_page_data=None, download_l1_data=False):
    """
    download cygnss data raw if files from PODAAC, the username and pass can be obtained from https://podaac-tools.jpl.nasa.gov/drive/
    Don't use this function directly, instead use download_cyg_rawif_files()

    :param data_year: data year (ex. 2019)
    :type data_year: int
    :param data_day: data day number (ex. 130)
    :type data_day: int
    :param sc_num: cygnss spacecraft number (1-8)
    :type sc_num: int
    :param out_folder: folder of the saved output (ex: /cygnss_data/L1/v2.1/2019/020)
    :type out_folder: str
    :param podaac_usr_pass: PODAAC user name and pass (usr name, pass)
    :type podaac_usr_pass: tuple of string
    :param day_page_data: page of the selected day, this to reduce the times the script requests the page from the website, default None
    :type day_page_data: list of str or None
    :param download_l1_data: when there is a Rawif data, download its L1 data with it, env $CYGNSS_L1_PATH var should point to the folder of L1 data.
    :type download_l1_data: bool
    :return: file names
    :rtype: list of str
    """
    _files_flag = np.zeros(2).astype(bool)  # check if both the data file and the metadata file are exist
    if not os.path.isdir(out_folder):
        raise ValueError('Invalid out_folder={} specified'.format(out_folder))

    cyg_folder_url = '{:s}/{:s}/{:s}/{:04d}/{:03d}/'.format(PODAAC_CYG_URL, 'L1', 'raw_if', data_year, data_day)
    cyg_files_url = list()
    if day_page_data is None:
        day_page_data = _get_cyg_page_data(cyg_folder_url, podaac_usr_pass)

    tag = 'cyg{:02d}_raw_if'.format(sc_num)
    endtag = '.bin'
    for item in day_page_data:
        if 'cyg' in item:
            try:
                ind = item.index(tag)
                item = item[ind:]
                end = item.index(endtag)
            except ValueError:
                pass
            else:
                file_name = item[:end + 4]
                if 'data' in file_name and not _files_flag[0]:  # only allow one data file name
                    _files_flag[0] = True
                    cyg_files_url.append('{:s}{:s}'.format(cyg_folder_url, file_name))
                elif 'meta' in file_name and not _files_flag[1]:  # only allow one metadata file name
                    _files_flag[1] = True
                    cyg_files_url.append('{:s}{:s}'.format(cyg_folder_url, file_name))

    if not cyg_files_url:  # if no file
        return None
    if not _files_flag.all():  # if only found the data file or the metadata file
        warnings.warn("couldn't find both data and the metadata files for year: {:04d}, day: {:03d}, sc: {:01d}".format(data_year, data_day, sc_num),
                      RuntimeWarning)
    out_files = list()
    for file_url in cyg_files_url:
        file = download_file(file_url, out_folder, auth=podaac_usr_pass)
        if file is not None:
            out_files.append(file)
    if download_l1_data:
        download_cyg_files(data_year, data_day, sc_num, podaac_usr=podaac_usr_pass[0], podaac_pass=podaac_usr_pass[1])
    return out_files


def download_rawif_cyg_files_between_date(st_date, end_date, list_sc_num=None, podaac_usr=None, podaac_pass=None,
                                          cygnss_l1_path=os.environ.get('CYGNSS_L1_PATH'), re_download=False, save_podaac_pass=True,
                                          download_l1_data=False):
    """
    download RAWIF CYGNSS data between two dates (including start and end date)

    :param st_date: start date
    :type st_date: dt.date
    :param end_date: end date
    :type end_date: dt.date
    :param list_sc_num: list of cygnss spacecraft numbers (1-8), if None will download all SCs
    :type list_sc_num: list or int or np.array or None
    :param podaac_usr: PODAAC user name. if None, it will ask you to enter it
    :type podaac_usr: str or None
    :param podaac_pass: PODAAC Drive API password. if None, it will ask you to enter it
    :type podaac_pass: str or None
    :param cygnss_l1_path: path of the cygnss L1 data (default: os.environ.get('CYGNSS_L1_PATH')), see description for more details
    :type cygnss_l1_path: str or None
    :param re_download: re-download the file if it exist?
    :type re_download: bool
    :param save_podaac_pass: save podaac username and pass in your system? (select False if you're using a shared or public PC)
    :type save_podaac_pass: bool
    :param download_l1_data: when there is a Rawif data, download its L1 data with it, env $CYGNSS_L1_PATH var should point to the folder of L1 data.
    :type download_l1_data: bool
    :return:
    """
    if list_sc_num is None:
        list_sc_num = np.arange(1, 9)
    elif np.size(list_sc_num) == 1:
        list_sc_num = [int(list_sc_num)]
    if cygnss_l1_path is None:
        raise ValueError("$CYGNSS_L1_PATH environment variable need to be set, or use cygnss_l1_path input parameter")

    # check if the folder name is not raw_if, if not, change the folder name
    folder_list = cygnss_l1_path.split(os.path.sep)
    if not folder_list[-1]:
        folder_list.pop(-1)
    if 'raw_if' not in folder_list[-1]:
        folder_list[-1] = 'raw_if'
    cygnss_l1_path = os.sep.join(folder_list)

    podaac_usr_pass = get_podaac_cred(save_pass=save_podaac_pass) if (podaac_usr is None or podaac_pass is None) else (podaac_usr, podaac_pass)

    num_days = (end_date - st_date).days + 1
    for iday in range(0, num_days):
        data_date = st_date + dt.timedelta(days=iday)
        data_year = data_date.year
        data_day = data_date.timetuple().tm_yday
        _download_single_day_rawif(data_year, data_day, list_sc_num, cygnss_l1_path, podaac_usr_pass, re_download, download_l1_data)


def download_cyg_rawif_files(data_year, list_data_day, list_sc_num=None, podaac_usr=None, podaac_pass=None,
                             cygnss_l1_path=os.environ.get('CYGNSS_L1_PATH'), re_download=False, save_podaac_pass=True, download_l1_data=False):
    """

    download the raw_if cygnss data,
    if cygnss_l1_path or os.environ.get('CYGNSS_L1_PATH') point to a folder with name not "raw_if", it will save the data in a raw_if folder in the
    parent dir.

    :param data_year: list of data years
    :type data_year:  int
    :param list_data_day: list of data days
    :type list_data_day: list or int or np.array
    :param list_sc_num: list of cygnss spacecraft numbers (1-8), if None will download all SCs
    :type list_sc_num: list or int or np.array or None
    :param podaac_usr: PODAAC user name. if None, it will ask you to enter it
    :type podaac_usr: str or None
    :param podaac_pass: PODAAC Drive API password. if None, it will ask you to enter it
    :type podaac_pass: str or None
    :param cygnss_l1_path: path of the cygnss L1 data (default: os.environ.get('CYGNSS_L1_PATH')), see description for more details
    :type cygnss_l1_path: str or None
    :param re_download: re-download the file if it exist?
    :type re_download: bool
    :param save_podaac_pass: save podaac username and pass in your system? (select False if you're using a shared or public PC)
    :type save_podaac_pass: bool
    :param download_l1_data: when there is a Rawif data, download its L1 data with it, env $CYGNSS_L1_PATH var should point to the folder of L1 data.
    :type download_l1_data: bool
    :return:
    """
    if np.size(list_data_day) == 1:
        list_data_day = [int(list_data_day)]
    if list_sc_num is None:
        list_sc_num = np.arange(1, 9)
    elif np.size(list_sc_num) == 1:
        list_sc_num = [int(list_sc_num)]
    if cygnss_l1_path is None:
        raise ValueError("$CYGNSS_L1_PATH environment variable need to be set, or use cygnss_l1_path input parameter")

    # check if the folder name is not raw_if, if not, change the folder name
    folder_list = cygnss_l1_path.split(os.path.sep)
    if not folder_list[-1]:
        folder_list.pop(-1)
    if 'raw_if' not in folder_list[-1]:
        folder_list[-1] = 'raw_if'
    cygnss_l1_path = os.sep.join(folder_list)

    podaac_usr_pass = get_podaac_cred(save_pass=save_podaac_pass) if (podaac_usr is None or podaac_pass is None) else (podaac_usr, podaac_pass)

    for data_day in list_data_day:
        _download_single_day_rawif(data_year, data_day, list_sc_num, cygnss_l1_path, podaac_usr_pass, re_download, download_l1_data)


def _download_single_day_rawif(data_year, data_day, list_sc_num, cygnss_l1_path, podaac_usr_pass, re_download, download_l1_data=False):
    """
    download Rawif data for a single day, don't use this function to download the files, instead use download_cyg_rawif_files() or
    download_rawif_cyg_files_between_date ()

    :param data_year: data year
    :type data_year:  int
    :param data_day: data day number
    :type data_day: int
    :param list_sc_num: list of cygnss spacecraft numbers (1-8)
    :type list_sc_num: list or int or np.array
    :param cygnss_l1_path: main path of the data
    :type cygnss_l1_path: str
    :param podaac_usr_pass: PODAAC user name and pass (usr name, pass)
    :type podaac_usr_pass: tuple of string
    :param re_download: re-download the file if it exist?
    :type re_download: bool
    :param download_l1_data: when there is a Rawif data, download its L1 data with it, env $CYGNSS_L1_PATH var should point to the folder of L1 data.
    :type download_l1_data: bool
    :return:
    """
    cyg_day_folder = os.path.join(cygnss_l1_path, '{:04d}'.format(data_year), '{:03d}'.format(data_day))
    cyg_day_folder_url = '{:s}/{:s}/{:s}/{:04d}/{:03d}/'.format(PODAAC_CYG_URL, 'L1', 'raw_if', data_year, data_day)
    day_page_data = _get_cyg_page_data(cyg_day_folder_url, podaac_usr_pass)
    if not os.path.isdir(cyg_day_folder):
        os.makedirs(cyg_day_folder, exist_ok=True)
    for sc_num in list_sc_num:
        cyg_files_name = get_cyg_rawif_files(cyg_day_folder, sc_num)
        if cyg_files_name is None:
            download_cyg_rawif(data_year, data_day, sc_num, cyg_day_folder, podaac_usr_pass=podaac_usr_pass, day_page_data=day_page_data,
                               download_l1_data=download_l1_data)

        elif re_download:
            for file_name in cyg_files_name:
                cyg_file_full_path = os.path.join(cygnss_l1_path, '{:04d}'.format(data_year), '{:03d}'.format(data_day), file_name)
                os.remove(cyg_file_full_path)
            download_cyg_rawif(data_year, data_day, sc_num, cyg_day_folder, podaac_usr_pass=podaac_usr_pass, day_page_data=day_page_data,
                               download_l1_data=download_l1_data)
        else:
            for file_name in cyg_files_name:
                print('{:s} file exist'.format(file_name))


def get_cyg_rawif_files(cyg_day_folder, sc_num):
    """
    check if the file exist and return the file name, if not exist return None.
    if exist it will return list of the files

    :param cyg_day_folder: cygnss day folder
    :type cyg_day_folder: str
    :param sc_num: spacecraft number
    :type sc_num: int
    :return: file name
    :rtype: str
    """
    _files_flag = np.zeros(2).astype(bool)
    result = []
    pattern = "cyg{:02d}*.bin".format(sc_num)
    for root, dirs, files in os.walk(cyg_day_folder):
        for name in files:
            if fnmatch.fnmatch(name, pattern):
                result.append(name)

    if len(result) == 0:
        return None
    else:
        files_name_list = list()
        for file_name in result:
            if 'data' in file_name:
                _files_flag[0] = True
                files_name_list.append(file_name)
            elif 'meta' in file_name:
                _files_flag[1] = True
                files_name_list.append(file_name)

    if not _files_flag.all():
        warnings.warn("couldn't find both data and the metadata files in {:s}, sc: {:d}, try to download both".format(cyg_day_folder, sc_num),
                      RuntimeWarning)
        return None

    return files_name_list


if __name__ == '__main__':
    down_start_date = dt.date(year=2020, month=8, day=4)
    down_end_date = dt.date(year=2020, month=8, day=4)
    download_rawif_cyg_files_between_date(down_start_date, down_end_date)
    download_cyg_rawif_files(data_year=2020, list_data_day=227)



import numpy as np
import os
try:
    import cygnsslib
except ImportError:
    import sys
    parent_dir_path = os.path.abspath(os.path.join(os.path.dirname(os.path.realpath(__file__)), os.pardir))
    sys.path.insert(0, parent_dir_path)
    import cygnsslib


if __name__ == '__main__':
    CYGNSS_L1_dir = os.environ["CYGNSS_L1_PATH"]
    year = 2018

    daylist = list(range(1, 365))

    radius = 5e3
    thesh_ddm_snr = 10.
    thesh_noise = 1
    max_grp_dist = 100
    point_names = ["Y1", "Y2", "Y3", "Y4", "Y5", "Y6", "Y7", "Y8", "Y9", "Y10", "Y11", "Y12", "Y13"]
    loc_list = np.array([[-34.62888, 145.84895], [-34.65478, 146.11028], [-34.621, 146.424], [-34.71943, 146.02003], [-34.72835, 146.29317], [-34.84262, 145.86692],
                         [-34.85183, 146.1153], [-34.84697, 146.41398], [-34.96777, 146.01632], [-35.00535, 146.30988], [-35.10975, 145.93553], [-35.0696, 146.16893],
                         [-35.09025, 146.30648]])
    for i_loc, loc in enumerate(loc_list):
        out_root = 'yanco_{:d}_{:s}'.format(year, point_names[i_loc])
        ref_pos = loc
        out_options = dict()
        out_options["plt_tag"] = point_names[i_loc]
        out_options["save_cvs"] = True
        out_high, out_low, out_ref = cygnsslib.write_sp_within_radius(CYGNSS_L1_dir, year, daylist, ref_pos, radius, out_root, thesh_ddm_snr,
                                                                      thesh_noise, out_options=out_options)
        out_kml = 'grp{:d}_{:s}'.format(max_grp_dist, out_high)
        cygnsslib.group_sp_within_distance(out_high, out_kml=out_kml, max_dist=max_grp_dist, save_csv=True)



from collections.abc import Iterable
from cygnsslib import get_list_ddm_info_from_kml
from cygnsslib.CygDdmId import CygDdmId, get_sample_id_of_different_cyg_version, cyg_environ_check
from netCDF4 import Dataset
import numpy as np
import os
import pandas as pd


def find_land_prod_sample_id_from_excel(xls_in, xls_out=None, start_col=1, st_row=1, out_sheet_suffix='', land_samp_col=None, timestamp_col=None):
    """

    Read the first sheet and the col. start_col to start_col+4.

    The output file is all the cells with new column: land_sample_zero_based

    :param xls_in: input Excel file
    :type xls_in: str
    :param xls_out: output Excel file name, if None add "_land_prod" to the file
    :type xls_out: str or None
    :param start_col: starting col. the function read start_col to start_col+4
    :type start_col: int
    :param st_row: Starting row, default 1 which is the header
    :type st_row: int
    :param out_sheet_suffix: suffix of the sheet name (NOT file name)
    :type out_sheet_suffix: str
    :param land_samp_col: col number of land_sample_zero_based, if None the default is start_col + 5
    :type land_samp_col: int or None
    :param timestamp_col: write the timestamp to tis col. if None the function will not write the timestamp
    :type timestamp_col: int or None
    :return: output file name
    :rtype: str
    """
    if xls_out is None:
        inxls_list = xls_in.split('.')
        xls_out = f'{inxls_list[0]:s}_land_prod.xlsx'  # car read xlsx and xls but write only to xls

    if not os.path.exists(xls_in):
        raise FileExistsError(f"Input Excel file doesn't exist, {xls_in:s}")
    df = pd.read_excel(xls_in)

    land_samp_list = []
    for i_ddm in df.itertuples():
        try:
            ddm_id = CygDdmId(None, i_ddm.year, i_ddm.day, i_ddm.flight_model, i_ddm.channel, i_ddm.sample_zero_based)
        except AttributeError:
            ddm_id = CygDdmId(None, i_ddm[start_col+1], i_ddm[start_col+2], i_ddm[start_col+3], i_ddm[start_col+4], i_ddm[start_col+5])
        try:
            ddm_id.fill_land_parameters()
        except Exception as e:
            ddm_id.land_samp_id = None
        land_samp_list.append(ddm_id.land_samp_id)

    if land_samp_col is None or land_samp_col < 0:
        try:
            land_samp_col = int(np.where('sample_zero_based' == df.columns)[0][0] + 1)
        except IndexError:
            land_samp_col = start_col + 5
    df.insert(land_samp_col, 'land_sample_zero_based', land_samp_list)
    df.to_excel(xls_out, index=False)


def get_ant_pattern_for_diff_versions(in_kml_xls, kml_cyg_ver, cyg_ver_list, out_xls):
    if in_kml_xls.endswith('.kml'):
        ddm_list = get_list_ddm_info_from_kml(in_kml_xls)
    else:
        try:
            in_df = pd.read_excel(in_kml_xls)
        except pd.Error.ParseError:
            in_df = pd.read_csv(in_kml_xls)
        ddm_list = []
        for i_row, row in in_df.iterrows():
            data = {'year': row.year, 'day': row.day, 'sc_num': row.sc_num, 'ch_id': row.ch_id}
            try:
                data['samp_id'] = row.sample_id
            except AttributeError:
                data['samp_id'] = row.samp_id
            ddm_list.append(data)

    cyg_environ_check()
    if kml_cyg_ver is not None:
        original_l1_path = os.path.join(os.environ.get("CYGNSS_PATH"), 'L1', kml_cyg_ver)
    else:
        original_l1_path = os.environ.get("CYGNSS_L1_PATH")
        kml_cyg_ver = original_l1_path.split(os.sep)[-1]
    if not isinstance(cyg_ver_list, Iterable):
        cyg_ver_list = [cyg_ver_list]
    cyg_ddm_id_list = []
    out_df = pd.DataFrame({'year': [],
                           'day': [],
                           'sc_num': [],
                           'ch_id': [],
                           'lat': [],
                           'lon': [],
                           'sp_inc_angle_deg': [],
                           'utc_time': [],
                           'utc_time_str': [],
                           'original_ver': [],
                           'org_ver_samp_id': [],
                           f'orig_ver_ant_pattern_db': [],
                           'original_ver_gps_eirp': []})
    for ddm_info in ddm_list:
        cyg_ddm_id = CygDdmId(None, ddm_info['year'], ddm_info['day'], ddm_info['sc_num'], ddm_info['ch_id'], ddm_info['samp_id'])
        cyg_ddm_id.fill_file_name(original_l1_path)
        cyg_ddm_id_list.append(cyg_ddm_id)
        cygnss_original_ver_file = os.path.join(original_l1_path, f'{cyg_ddm_id.year:04d}', f'{cyg_ddm_id.day:03d}', cyg_ddm_id.file_name)
        ds_cyg = Dataset(cygnss_original_ver_file, 'r')
        ds_cyg.set_auto_maskandscale(False)
        orig_ant_patt = float(ds_cyg.variables["sp_rx_gain"][cyg_ddm_id.samp_id, cyg_ddm_id.ch_id - 1])
        ddm_time = pd.Timedelta(int(ds_cyg.variables['ddm_timestamp_utc'][cyg_ddm_id.samp_id] * 1e9), 'ns') + pd.Timestamp(ds_cyg.time_coverage_start)
        lat = float(ds_cyg.variables['sp_lat'][cyg_ddm_id.samp_id, cyg_ddm_id.ch_id - 1])
        lon = float(ds_cyg.variables['sp_lon'][cyg_ddm_id.samp_id, cyg_ddm_id.ch_id - 1])
        gps_eirp = float(ds_cyg.variables['gps_eirp'][cyg_ddm_id.samp_id, cyg_ddm_id.ch_id - 1])
        inc_angle = float(ds_cyg.variables['sp_inc_angle'][cyg_ddm_id.samp_id, cyg_ddm_id.ch_id - 1])
        if lon > 180:
            lon -= 360.0
        ds_cyg.close()
        out_df = out_df.append(pd.DataFrame({'year': [ddm_info['year']],
                                             'day': [ddm_info['day']],
                                             'sc_num': [ddm_info['sc_num']],
                                             'ch_id': [ddm_info['ch_id']],
                                             'lat': [lat],
                                             'lon': [lon],
                                             'sp_inc_angle_deg': [inc_angle],
                                             'utc_time': [ddm_time.tz_convert(None)],  # make it tz unaware for Excel
                                             'utc_time_str': [str(ddm_time)],
                                             'original_ver': [kml_cyg_ver],
                                             'org_ver_samp_id': [ddm_info['samp_id']],
                                             'orig_ver_ant_pattern_db': [orig_ant_patt],
                                             'original_ver_gps_eirp': [gps_eirp]
                                             }), ignore_index=True)
    for iver, cyg_ver in enumerate(cyg_ver_list, 1):
        cur_ver_l1_path = os.path.join(os.environ.get("CYGNSS_PATH"), 'L1', cyg_ver)
        ant_patt_list = []
        curr_ver_samp_id = []
        gps_eirp_list = []
        for i_ddm, cyg_ddm_id in enumerate(cyg_ddm_id_list):
            new_cyg_ddm_id = get_sample_id_of_different_cyg_version(cyg_ddm_id, cyg_ver, original_cyg_ver=kml_cyg_ver)
            cygnss_cur_ver_file = os.path.join(cur_ver_l1_path, f'{new_cyg_ddm_id.year:04d}', f'{new_cyg_ddm_id.day:03d}', new_cyg_ddm_id.file_name)
            ds_cyg = Dataset(cygnss_cur_ver_file, 'r')
            ds_cyg.set_auto_maskandscale(False)
            ant_patt = float(ds_cyg.variables["sp_rx_gain"][new_cyg_ddm_id.samp_id, new_cyg_ddm_id.ch_id - 1])
            ddm_time_ns_rltv_tcs = int(ds_cyg.variables['ddm_timestamp_utc'][new_cyg_ddm_id.samp_id] * 1e9)
            ddm_time = pd.Timedelta(ddm_time_ns_rltv_tcs, 'ns') + pd.Timestamp(ds_cyg.time_coverage_start)
            gps_eirp_list.append(float(ds_cyg.variables['gps_eirp'][cyg_ddm_id.samp_id, cyg_ddm_id.ch_id - 1]))
            if ddm_time.tz_convert(None) - out_df.utc_time[i_ddm] > pd.Timedelta(2, 'ns'):
                raise RuntimeError(f'There is something wrong with the code. DDM time of original version is different from the time of this version'
                                   f'\nddm time original ver:{out_df.utc_time[i_ddm]}, this ver{ddm_time}')
            ds_cyg.close()
            ant_patt_list.append(ant_patt)
            curr_ver_samp_id.append(new_cyg_ddm_id.samp_id)
        out_df.insert(len(out_df.columns), f'ver_num{iver}', [cyg_ver] * len(ant_patt_list))
        out_df.insert(len(out_df.columns), f'samp_id_num{iver}', curr_ver_samp_id)
        out_df.insert(len(out_df.columns), f'ant_patt_num{iver}_db', ant_patt_list)
        out_df.insert(len(out_df.columns), f'gps_eirp_num{iver}', gps_eirp_list)
    out_df.to_excel(out_xls, index=True)
    return out_xls


if __name__ == '__main__':
    # xls_in = 'SLV_Z4_thawed_2019.xlsx'
    # find_land_prod_sample_id_from_excel(xls_in, xls_out=None, start_col=1, out_sheet_suffix='', timestamp_col=0)
    # xls_in = 'SLV_Z1_thawed_2019.xlsx'
    # find_land_prod_sample_id_from_excel(xls_in, xls_out=None, st_row=1, start_col=1, out_sheet_suffix='', timestamp_col=0)
    kml_file_path = 'z1_test_v3_samples.xlsx'
    ant_comp_xls = 'z1_test_v3_samples_antt.xlsx'
    get_ant_pattern_for_diff_versions(kml_file_path, kml_cyg_ver='v3.0', cyg_ver_list=['v2.1'], out_xls=ant_comp_xls)
    kml_file_path = '/media/amer/myfiles/cygnss/kml_files/Z1/z1_yr19_aft_aug_above_thresh.kml'
    ant_comp_xls = 'v2v3_ant_com.xlsx'
    get_ant_pattern_for_diff_versions(kml_file_path, kml_cyg_ver='v3.0', cyg_ver_list=['v2.1'], out_xls=ant_comp_xls)


import numpy as np
import os
try:
    from cygnsslib import srtm_download
except ImportError:
    import sys
    parent_dir_path = os.path.abspath(os.path.join(os.path.dirname(os.path.realpath(__file__)), os.pardir))
    sys.path.insert(0, parent_dir_path)
    from cygnsslib import srtm_download

if __name__ == '__main__':
    lat = np.array([21, 22, 20])
    lon = [29, 30]
    srtm_download.download_srtm_ght_files(lat, lon)
